import sys
from pathlib import Path
from magcopy import Actions, EmptyClipboardError, InvalidPathError, clipboard
from magcopy.path import paste

usage = """\
Usage: magcopy <action> <path/file>

Actions:
    copy   - Copy folder or file to clipboard
    move   - Add folder or file to clipboard to move
    paste  - Paste folder or file from clipboard to current folder or specified folder

Options:
    --help  - Display this usage message.

Note:
    This program does not alter the contents of the system's default clipboard. It maintains its own clipboard for managing copied and moved items.

Example:
    magcopy copy /path/to/source/folder
    magcopy paste /path/to/destination/folder

    magcopy move /path/to/source/file
    magcopy paste /path/to/destination/folder

    magcopy copy /path/to/source/file --clip
"""


def get_arg_path() -> str:
    if len(sys.argv) == 0:
        print("Error: Please provide the path for the file or folder.")
        sys.exit(1)

    path = sys.argv.pop(0)
    return Path(path).resolve().as_posix()


def execute_path_action(action: str) -> None:
    path = get_arg_path()

    try:
        clipboard.add_to_clipboard(action, path)
    except InvalidPathError as error:
        print(error)
        sys.exit(1)


def main():
    sys.argv = sys.argv[1:]

    if "--help" in sys.argv or len(sys.argv) == 0:
        print(usage)
        sys.exit(0)

    command = sys.argv.pop(0)
    if command == "copy":
        execute_path_action(Actions.COPY)

    elif command == "move":
        execute_path_action(Actions.MOVE)

    elif command == "paste":
        path = get_arg_path()

        try:
            paste(path)
        except EmptyClipboardError as error:
            print(error)
            sys.exit(1)

    else:
        print(
            f"Error: Unknown or invalid command '{command}'. Use 'magcopy --help' to view the usage guide."
        )
        sys.exit(1)


if __name__ == "__mai__":
    main()



import pytest
from pathlib import Path
from magcopy import InvalidPathError, ACTION_FILE, CLIPBOARD_FILE
from magcopy.clipboard import (
    add_to_clipboard,
    get_action,
    read_from_clipboard,
    read_all_from_clipboard,
)


@pytest.fixture
def cleanup_clipboard_files():
    # Clean up clipboard files after each test
    yield
    ACTION_FILE.unlink(missing_ok=True)
    CLIPBOARD_FILE.unlink(missing_ok=True)


def test_add_to_clipboard(cleanup_clipboard_files):
    path1 = Path(__file__).parent.joinpath("__init__.py").resolve()
    path2 = Path(__file__).parent.joinpath("test_clipboard.py").resolve()

    print(path2.is_file())

    add_to_clipboard("copy", path1)
    assert get_action() == "copy"
    assert read_from_clipboard() == Path(path1)
    assert read_all_from_clipboard() == [Path(path1)]

    add_to_clipboard("move", path2)
    assert get_action() == "move"
    assert read_from_clipboard() == Path(path2)
    assert read_all_from_clipboard() == [Path(path2), Path(path1)]


def test_add_to_clipboard_invalid_path(cleanup_clipboard_files):
    with pytest.raises(InvalidPathError):
        add_to_clipboard("copy", "nonexistent_file.txt")


def test_read_from_clipboard(cleanup_clipboard_files):
    path = Path(__file__).parent.joinpath("__init__.py").resolve()
    add_to_clipboard("copy", path)

    assert read_from_clipboard() == Path(path)


def test_read_all_from_clipboard(cleanup_clipboard_files):
    path1 = Path(__file__).parent.joinpath("__init__.py").resolve()
    path2 = Path(__file__).parent.joinpath("test_clipboard.py").resolve()
    add_to_clipboard("copy", path1)
    add_to_clipboard("move", path2)

    assert read_all_from_clipboard() == [Path(path2), Path(path1)]


def test_get_action(cleanup_clipboard_files):
    assert get_action() == ""  # No action initially

    add_to_clipboard("copy", Path(__file__).resolve())
    assert get_action() == "copy"

    add_to_clipboard("move", Path(__file__).parent.resolve())
    assert get_action() == "move"



import shutil
from pathlib import Path
from magcopy import (
    Actions,
    InvalidActionError,
    InvalidPathError,
    OutputIsNonDirectoryError,
    PathLike,
)
from magcopy.clipboard import clear_clipboard, get_action, read_from_clipboard


def __copy(input_path: Path, output_path: Path) -> None:
    if output_path.is_file():
        raise OutputIsNonDirectoryError(
            f"Destination '{output_path}' is a file, not a directory."
        )

    if input_path.is_file():
        shutil.copy(input_path, output_path)
        print(f'File successfully copied to "{output_path}".')

    elif input_path.is_dir():
        shutil.copytree(input_path, output_path.joinpath(input_path.name))
        print(f'Folder successfully copied to "{output_path}".')


def __move(input_path: Path, output_path: Path):
    if output_path.is_file():
        raise OutputIsNonDirectoryError(
            f"Destination '{output_path}' is a file, not a directory."
        )

    if input_path.is_file():
        shutil.move(input_path, output_path)
        print(f'File successfully moved to "{output_path}".')

    elif input_path.is_dir():
        shutil.move(input_path, output_path.joinpath(input_path.name))
        print(f'Folder successfully moved to "{output_path}".')


def paste(path: PathLike):
    input_path = read_from_clipboard()
    output_path = Path(path) if isinstance(path, str) else path
    action = get_action()

    if not output_path.is_file() and not output_path.is_dir():
        raise InvalidPathError("Error: Please provide a valid path.")

    if action == Actions.COPY:
        __copy(input_path, output_path)
    elif action == Actions.MOVE:
        __move(input_path, output_path)
    else:
        raise InvalidActionError(f"Error: Unknown or invalid action '{action}'")

    clear_clipboard()



from pathlib import Path
from magcopy import (
    EmptyClipboardError,
    InvalidPathError,
    PathLike,
    MAX_ITEMS_IN_CLIPBOARD,
    ACTION_FILE,
    CLIPBOARD_FILE,
)


def __create_files_if_not_exists() -> None:
    for file in [CLIPBOARD_FILE, ACTION_FILE]:
        if not file.is_file():
            file.touch()


def get_action() -> str:
    __create_files_if_not_exists()

    return ACTION_FILE.read_text(encoding="utf-8").strip().lower()


def add_to_clipboard(action: str, path: PathLike) -> None:
    __create_files_if_not_exists()

    path_obj = Path(path) if isinstance(path, str) else path
    if not path_obj.is_file() and not path_obj.is_dir():
        raise InvalidPathError("Error: Please provide a valid path.")

    clipboard_items: list[str] = [
        item.strip()
        for item in CLIPBOARD_FILE.read_text(encoding="utf-8").split("\n")
        if item != ""
    ]
    clipboard_items.insert(0, path_obj.as_posix())
    if len(clipboard_items) > MAX_ITEMS_IN_CLIPBOARD:
        clipboard_items.pop()

    CLIPBOARD_FILE.write_text("\n".join(clipboard_items), encoding="utf-8")
    ACTION_FILE.write_text(action.lower(), encoding="utf-8")


def read_from_clipboard() -> Path:
    __create_files_if_not_exists()

    clipboard_items: list[str] = [
        item.strip()
        for item in CLIPBOARD_FILE.read_text().split("\n")
        if item != ""
    ]

    if len(clipboard_items) == 0:
        raise EmptyClipboardError(
            "Error: The clipboard is empty. Use the 'copy' or 'move' command before 'paste'."
        )

    return Path(clipboard_items[0])


def read_all_from_clipboard() -> list[Path]:
    __create_files_if_not_exists()

    clipboard_items: list[str] = [
        item.strip()
        for item in CLIPBOARD_FILE.read_text().split("\n")
        if item != ""
    ]

    return [Path(item) for item in clipboard_items]


def clear_clipboard():
    ACTION_FILE.write_text("")
    CLIPBOARD_FILE.write_text("")



from pathlib import Path
from typing import Union

PathLike = Union[str, Path]

MAX_ITEMS_IN_CLIPBOARD = 10
CLIPBOARD_FILE = Path(__file__).parent.parent.joinpath(".clipboard").resolve()
ACTION_FILE = Path(__file__).parent.parent.joinpath(".action").resolve()

__version__ = "0.0.3.1"


class Actions:
    COPY = "copy"
    MOVE = "move"
    PASTE = "paste"


class InvalidPathError(Exception):
    pass


class InvalidActionError(Exception):
    pass


class OutputIsNonDirectoryError(Exception):
    pass


class EmptyClipboardError(Exception):
    pass
